# !/usr/bin/env python
# coding:utf-8

# Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.

# Based on the Apache License 2.0 open source protocol.

__author__ = 'quinn.7@foxmail.com'


class BrowserType(object):
    """
    浏览器类型
    """

    CHROME = "BROWSER_TYPE_CHROME"
    """ 谷歌浏览器 """
